# $ANTLR 3.1.2 /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g 2012-10-23 16:08:43

import sys
from cadbiom.antlr3 import *
from cadbiom.antlr3.compat import set, frozenset

from cadbiom.antlr3.tree import *


from cadbiom.models.biosignal.sig_expr import *
import string



# for convenience in actions
HIDDEN = BaseRecognizer.HIDDEN

# token types
F=17
DEF=19
SYNC=9
LETTER=32
DEFAULT=6
T=16
CHG=26
NOT=15
MINUS=29
EVENT=8
AND=13
EOF=-1
MUL=30
CONSTR=18
INC=11
SCOL=27
WS=4
EXC=10
EXP=31
SEQ=12
NOTEG=21
WHEN=7
COM=25
OR=14
DOL=24
IDENT=34
PLUS=28
DIGIT=33
COMMENT=5
PD=23
EG=20
PG=22

# token names
tokenNames = [
    "<invalid>", "<EOR>", "<DOWN>", "<UP>",
    "WS", "COMMENT", "DEFAULT", "WHEN", "EVENT", "SYNC", "EXC", "INC", "SEQ",
    "AND", "OR", "NOT", "T", "F", "CONSTR", "DEF", "EG", "NOTEG", "PG",
    "PD", "DOL", "COM", "CHG", "SCOL", "PLUS", "MINUS", "MUL", "EXP", "LETTER",
    "DIGIT", "IDENT"
]




class sigexpr_compiler(Parser):
    grammarFileName = "/home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g"
    antlr_version = version_str_to_tuple("3.1.2")
    antlr_version_str = "3.1.2"
    tokenNames = tokenNames

    def __init__(self, input, state=None):
        if state is None:
            state = RecognizerSharedState()

        Parser.__init__(self, input, state)


        self.dfa1 = self.DFA1(
            self, 1,
            eot = self.DFA1_eot,
            eof = self.DFA1_eof,
            min = self.DFA1_min,
            max = self.DFA1_max,
            accept = self.DFA1_accept,
            special = self.DFA1_special,
            transition = self.DFA1_transition
            )

        self.dfa3 = self.DFA3(
            self, 3,
            eot = self.DFA3_eot,
            eof = self.DFA3_eof,
            min = self.DFA3_min,
            max = self.DFA3_max,
            accept = self.DFA3_accept,
            special = self.DFA3_special,
            transition = self.DFA3_transition
            )

        self.dfa7 = self.DFA7(
            self, 7,
            eot = self.DFA7_eot,
            eof = self.DFA7_eof,
            min = self.DFA7_min,
            max = self.DFA7_max,
            accept = self.DFA7_accept,
            special = self.DFA7_special,
            transition = self.DFA7_transition
            )




        self.state_events = []
        self.free_clocks = []
        self.state_only = False
        self.state_only_in_bool = False
        self.catch_free_clocks = False
        self.deep = -1
        self.message = ""
        self.error_reporter = None





        self._adaptor = CommonTreeAdaptor()



    def getTreeAdaptor(self):
        return self._adaptor

    def setTreeAdaptor(self, adaptor):
        self._adaptor = adaptor

    adaptor = property(getTreeAdaptor, setTreeAdaptor)


    def set_error_reporter(self, err):
        self.error_reporter = err

    def displayRecognitionError(self, tokenNames, re):
           hdr = self.getErrorHeader(re)
           msg = self.getErrorMessage(re, tokenNames)
           self.error_reporter.display('sig_exp ->'+hdr+msg+self.message)


    def displayExceptionMessage(self, e):
      msg = self.getErrorMessage(self, e, tokenNames)
      self.error_reporter.display('sig_exp ->'+msg)

    # semantic checks for compilers
    def check_ident(self, symbol_table, st_only, free_clock, deep, message, name):
        """
        Check if  name  is declared  (with state/input type if st_only = True)
        @return: a SigIdentExpr
        """
        try:
          name = name.encode("utf-8")
          type, s_deep = symbol_table[name]
          # for condition compiler
          if st_only:
            if (not (type == "state" or type=="input")):
              self.error_reporter.display("type error -> " + name + " is not a state (" +type+")"+ message)
          else:
            if deep >= 0:
              if s_deep >= deep:
                 self.error_reporter.display("type error -> " + name + " not declared in a surrounding macro state"+ message)
        except KeyError:
          if free_clock and not st_only:
            self.free_clocks.append(name)
          else:
            self.error_reporter.display("dec -> Undeclared event or state:" + name + message)
        return SigIdentExpr(name)

    def check_updown(self, symbol_table, id, mode):
        """
        This function introduce new signals: state> or state<
        """
        id  = id.encode("utf-8")
        try:
            att = symbol_table[id]
            type = att[0]
            s_deep = att[1]
        except KeyError:
            self.error_reporter.display('dec -> Undeclared state in variation:' + id)
            type = "error"
        if type == "state":
            if mode == 1: #up
                name = id+">"
            else: #down
                name = id+"<"
            self.state_events.append(name)
            return SigIdentExpr(name)
        else:
            self.error_reporter.display('type error -> Up and Down can only be derived from a state: '+id)

    def  check_change(self, symbol_table, id):
        id  = id.encode("utf-8")
        try:
            att = symbol_table[id]
            type = att[0]
            s_deep = att[1]
        except KeyError:
            self.error_reporter.display('dec -> Undeclared signal in variation:' + id)
            type = "error"
        if type == "state":
            refresh_expr = SigIdentExpr(id)
            st_expr = SigIdentExpr(id)
            return SigWhenExpr(SigConstExpr(True),SigDiffExpr(st_expr, refresh_expr))
        else:
            self.error_reporter.display('type error -> Change can only be derived  from a state: '+id)

    def check_sync(self, lexp):
        # type checking is done in expressions
        return SigConstraintExpr(SigConstraintExpr.SYNCHRO, lexp)

    def check_exclus(self, lexp):
        return SigConstraintExpr(SigConstraintExpr.EXCLU, lexp)


    def check_included(self, exp1,exp2):
        return SigConstraintExpr(SigConstraintExpr.INCL, [exp1,exp2])



    class sig_expression_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_expression"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:163:1: sig_expression[tab_symb] returns [exp] : (exp1= sig_expression1[tab_symb] DOL | exp2= sig_constraint[tab_symb] DOL );
    def sig_expression(self, tab_symb):

        retval = self.sig_expression_return()
        retval.start = self.input.LT(1)

        root_0 = None

        DOL1 = None
        DOL2 = None
        exp1 = None

        exp2 = None


        DOL1_tree = None
        DOL2_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:164:9: (exp1= sig_expression1[tab_symb] DOL | exp2= sig_constraint[tab_symb] DOL )
                alt1 = 2
                alt1 = self.dfa1.predict(self.input)
                if alt1 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:164:11: exp1= sig_expression1[tab_symb] DOL
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_sig_expression1_in_sig_expression148)
                    exp1 = self.sig_expression1(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, exp1.tree)
                    DOL1=self.match(self.input, DOL, self.FOLLOW_DOL_in_sig_expression151)

                    DOL1_tree = self._adaptor.createWithPayload(DOL1)
                    self._adaptor.addChild(root_0, DOL1_tree)

                    #action start
                    retval.exp = ((exp1 is not None) and [exp1.exp] or [None])[0]
                    #action end


                elif alt1 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:166:11: exp2= sig_constraint[tab_symb] DOL
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_sig_constraint_in_sig_expression181)
                    exp2 = self.sig_constraint(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, exp2.tree)
                    DOL2=self.match(self.input, DOL, self.FOLLOW_DOL_in_sig_expression184)

                    DOL2_tree = self._adaptor.createWithPayload(DOL2)
                    self._adaptor.addChild(root_0, DOL2_tree)

                    #action start
                    retval.exp = ((exp2 is not None) and [exp2.exp] or [None])[0]
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_expression"

    class sig_expression1_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_expression1"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:170:1: sig_expression1[tab_symb] returns [exp] : (exp1= sig_exp[tab_symb] ( DEFAULT exp2= sig_exp[tab_symb] )* | );
    def sig_expression1(self, tab_symb):

        retval = self.sig_expression1_return()
        retval.start = self.input.LT(1)

        root_0 = None

        DEFAULT3 = None
        exp1 = None

        exp2 = None


        DEFAULT3_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:171:9: (exp1= sig_exp[tab_symb] ( DEFAULT exp2= sig_exp[tab_symb] )* | )
                alt3 = 2
                alt3 = self.dfa3.predict(self.input)
                if alt3 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:171:11: exp1= sig_exp[tab_symb] ( DEFAULT exp2= sig_exp[tab_symb] )*
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_sig_exp_in_sig_expression1230)
                    exp1 = self.sig_exp(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, exp1.tree)
                    #action start
                    retval.exp = ((exp1 is not None) and [exp1.exp] or [None])[0]
                    #action end
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:172:11: ( DEFAULT exp2= sig_exp[tab_symb] )*
                    while True: #loop2
                        alt2 = 2
                        LA2_0 = self.input.LA(1)

                        if (LA2_0 == DEFAULT) :
                            alt2 = 1


                        if alt2 == 1:
                            # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:172:12: DEFAULT exp2= sig_exp[tab_symb]
                            pass
                            DEFAULT3=self.match(self.input, DEFAULT, self.FOLLOW_DEFAULT_in_sig_expression1247)

                            DEFAULT3_tree = self._adaptor.createWithPayload(DEFAULT3)
                            root_0 = self._adaptor.becomeRoot(DEFAULT3_tree, root_0)

                            self._state.following.append(self.FOLLOW_sig_exp_in_sig_expression1252)
                            exp2 = self.sig_exp(tab_symb)

                            self._state.following.pop()
                            self._adaptor.addChild(root_0, exp2.tree)
                            #action start
                            retval.exp = SigDefaultExpr(retval.exp, ((exp2 is not None) and [exp2.exp] or [None])[0])
                            #action end


                        else:
                            break #loop2




                elif alt3 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:176:11:
                    pass
                    root_0 = self._adaptor.nil()

                    #action start
                    retval.exp = SigConstExpr(True)
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_expression1"

    class sig_exp_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_exp"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:179:1: sig_exp[tab_symb] returns [exp] : exp1= sig_bool[tab_symb] ( WHEN exp2= sig_bool[tab_symb] )* ;
    def sig_exp(self, tab_symb):

        retval = self.sig_exp_return()
        retval.start = self.input.LT(1)

        root_0 = None

        WHEN4 = None
        exp1 = None

        exp2 = None


        WHEN4_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:180:9: (exp1= sig_bool[tab_symb] ( WHEN exp2= sig_bool[tab_symb] )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:180:12: exp1= sig_bool[tab_symb] ( WHEN exp2= sig_bool[tab_symb] )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_bool_in_sig_exp338)
                exp1 = self.sig_bool(tab_symb)

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.exp = ((exp1 is not None) and [exp1.exp] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:181:11: ( WHEN exp2= sig_bool[tab_symb] )*
                while True: #loop4
                    alt4 = 2
                    LA4_0 = self.input.LA(1)

                    if (LA4_0 == WHEN) :
                        alt4 = 1


                    if alt4 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:181:12: WHEN exp2= sig_bool[tab_symb]
                        pass
                        WHEN4=self.match(self.input, WHEN, self.FOLLOW_WHEN_in_sig_exp355)

                        WHEN4_tree = self._adaptor.createWithPayload(WHEN4)
                        root_0 = self._adaptor.becomeRoot(WHEN4_tree, root_0)

                        #action start
                        st_only_save = self.state_only; self.state_only = self.state_only_in_bool
                        #action end
                        self._state.following.append(self.FOLLOW_sig_bool_in_sig_exp389)
                        exp2 = self.sig_bool(tab_symb)

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.exp = SigWhenExpr(retval.exp, ((exp2 is not None) and [exp2.exp] or [None])[0])
                        self.state_only = st_only_save
                        #action end


                    else:
                        break #loop4





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_exp"

    class sig_bool_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_bool"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:190:1: sig_bool[tab_symb] returns [exp] : exp1= sig_bool_and[tab_symb] ( OR exp2= sig_bool_and[tab_symb] )* ;
    def sig_bool(self, tab_symb):

        retval = self.sig_bool_return()
        retval.start = self.input.LT(1)

        root_0 = None

        OR5 = None
        exp1 = None

        exp2 = None


        OR5_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:191:9: (exp1= sig_bool_and[tab_symb] ( OR exp2= sig_bool_and[tab_symb] )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:191:11: exp1= sig_bool_and[tab_symb] ( OR exp2= sig_bool_and[tab_symb] )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_bool_and_in_sig_bool471)
                exp1 = self.sig_bool_and(tab_symb)

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.exp = ((exp1 is not None) and [exp1.exp] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:192:11: ( OR exp2= sig_bool_and[tab_symb] )*
                while True: #loop5
                    alt5 = 2
                    LA5_0 = self.input.LA(1)

                    if (LA5_0 == OR) :
                        alt5 = 1


                    if alt5 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:192:12: OR exp2= sig_bool_and[tab_symb]
                        pass
                        OR5=self.match(self.input, OR, self.FOLLOW_OR_in_sig_bool487)

                        OR5_tree = self._adaptor.createWithPayload(OR5)
                        root_0 = self._adaptor.becomeRoot(OR5_tree, root_0)

                        self._state.following.append(self.FOLLOW_sig_bool_and_in_sig_bool492)
                        exp2 = self.sig_bool_and(tab_symb)

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.exp = SigSyncBinExpr("or", retval.exp, ((exp2 is not None) and [exp2.exp] or [None])[0])
                        #action end


                    else:
                        break #loop5





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_bool"

    class sig_bool_and_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_bool_and"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:197:1: sig_bool_and[tab_symb] returns [exp] : exp1= sig_primary[tab_symb] ( AND exp2= sig_primary[tab_symb] )* ;
    def sig_bool_and(self, tab_symb):

        retval = self.sig_bool_and_return()
        retval.start = self.input.LT(1)

        root_0 = None

        AND6 = None
        exp1 = None

        exp2 = None


        AND6_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:198:9: (exp1= sig_primary[tab_symb] ( AND exp2= sig_primary[tab_symb] )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:198:11: exp1= sig_primary[tab_symb] ( AND exp2= sig_primary[tab_symb] )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_primary_in_sig_bool_and561)
                exp1 = self.sig_primary(tab_symb)

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.exp = ((exp1 is not None) and [exp1.exp] or [None])[0]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:199:11: ( AND exp2= sig_primary[tab_symb] )*
                while True: #loop6
                    alt6 = 2
                    LA6_0 = self.input.LA(1)

                    if (LA6_0 == AND) :
                        alt6 = 1


                    if alt6 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:199:12: AND exp2= sig_primary[tab_symb]
                        pass
                        AND6=self.match(self.input, AND, self.FOLLOW_AND_in_sig_bool_and577)

                        AND6_tree = self._adaptor.createWithPayload(AND6)
                        root_0 = self._adaptor.becomeRoot(AND6_tree, root_0)

                        self._state.following.append(self.FOLLOW_sig_primary_in_sig_bool_and582)
                        exp2 = self.sig_primary(tab_symb)

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.exp = SigSyncBinExpr("and",retval.exp, ((exp2 is not None) and [exp2.exp] or [None])[0])
                        #action end


                    else:
                        break #loop6





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_bool_and"

    class sig_primary_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_primary"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:204:1: sig_primary[tab_symb] returns [exp] : ( NOT nexp= sig_primary[tab_symb] | cexp= sig_constant | EVENT PG exps= sig_expression1[tab_symb] PD | WHEN PG expw= sig_expression1[tab_symb] PD | i7= IDENT UP | i8= IDENT DOWN | id2= IDENT | PG expse= sig_expression1[tab_symb] PD );
    def sig_primary(self, tab_symb):

        retval = self.sig_primary_return()
        retval.start = self.input.LT(1)

        root_0 = None

        i7 = None
        i8 = None
        id2 = None
        NOT7 = None
        EVENT8 = None
        PG9 = None
        PD10 = None
        WHEN11 = None
        PG12 = None
        PD13 = None
        UP14 = None
        DOWN15 = None
        PG16 = None
        PD17 = None
        nexp = None

        cexp = None

        exps = None

        expw = None

        expse = None


        i7_tree = None
        i8_tree = None
        id2_tree = None
        NOT7_tree = None
        EVENT8_tree = None
        PG9_tree = None
        PD10_tree = None
        WHEN11_tree = None
        PG12_tree = None
        PD13_tree = None
        UP14_tree = None
        DOWN15_tree = None
        PG16_tree = None
        PD17_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:205:9: ( NOT nexp= sig_primary[tab_symb] | cexp= sig_constant | EVENT PG exps= sig_expression1[tab_symb] PD | WHEN PG expw= sig_expression1[tab_symb] PD | i7= IDENT UP | i8= IDENT DOWN | id2= IDENT | PG expse= sig_expression1[tab_symb] PD )
                alt7 = 8
                alt7 = self.dfa7.predict(self.input)
                if alt7 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:205:11: NOT nexp= sig_primary[tab_symb]
                    pass
                    root_0 = self._adaptor.nil()

                    NOT7=self.match(self.input, NOT, self.FOLLOW_NOT_in_sig_primary656)

                    NOT7_tree = self._adaptor.createWithPayload(NOT7)
                    root_0 = self._adaptor.becomeRoot(NOT7_tree, root_0)

                    self._state.following.append(self.FOLLOW_sig_primary_in_sig_primary661)
                    nexp = self.sig_primary(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, nexp.tree)
                    #action start
                    retval.exp = SigNotExpr(((nexp is not None) and [nexp.exp] or [None])[0])
                    #action end


                elif alt7 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:208:11: cexp= sig_constant
                    pass
                    root_0 = self._adaptor.nil()

                    self._state.following.append(self.FOLLOW_sig_constant_in_sig_primary704)
                    cexp = self.sig_constant()

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, cexp.tree)
                    #action start
                    retval.exp = ((cexp is not None) and [cexp.exp] or [None])[0]
                    #action end


                elif alt7 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:211:11: EVENT PG exps= sig_expression1[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    EVENT8=self.match(self.input, EVENT, self.FOLLOW_EVENT_in_sig_primary751)

                    EVENT8_tree = self._adaptor.createWithPayload(EVENT8)
                    self._adaptor.addChild(root_0, EVENT8_tree)

                    PG9=self.match(self.input, PG, self.FOLLOW_PG_in_sig_primary753)
                    self._state.following.append(self.FOLLOW_sig_expression1_in_sig_primary758)
                    exps = self.sig_expression1(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, exps.tree)
                    PD10=self.match(self.input, PD, self.FOLLOW_PD_in_sig_primary761)
                    #action start
                    retval.exp = SigEventExpr(((exps is not None) and [exps.exp] or [None])[0])
                    #action end


                elif alt7 == 4:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:214:11: WHEN PG expw= sig_expression1[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    WHEN11=self.match(self.input, WHEN, self.FOLLOW_WHEN_in_sig_primary798)

                    WHEN11_tree = self._adaptor.createWithPayload(WHEN11)
                    self._adaptor.addChild(root_0, WHEN11_tree)

                    PG12=self.match(self.input, PG, self.FOLLOW_PG_in_sig_primary800)
                    self._state.following.append(self.FOLLOW_sig_expression1_in_sig_primary805)
                    expw = self.sig_expression1(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, expw.tree)
                    PD13=self.match(self.input, PD, self.FOLLOW_PD_in_sig_primary808)
                    #action start
                    retval.exp = SigWhenExpr(SigConstExpr(True),((expw is not None) and [expw.exp] or [None])[0])
                    #action end


                elif alt7 == 5:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:217:11: i7= IDENT UP
                    pass
                    root_0 = self._adaptor.nil()

                    i7=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sig_primary838)

                    i7_tree = self._adaptor.createWithPayload(i7)
                    self._adaptor.addChild(root_0, i7_tree)

                    UP14=self.match(self.input, 3, self.FOLLOW_3_in_sig_primary840)

                    UP14_tree = self._adaptor.createWithPayload(UP14)
                    self._adaptor.addChild(root_0, UP14_tree)

                    #action start
                    retval.exp = self.check_updown(tab_symb, i7.text, 1)
                    #action end


                elif alt7 == 6:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:220:11: i8= IDENT DOWN
                    pass
                    root_0 = self._adaptor.nil()

                    i8=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sig_primary870)

                    i8_tree = self._adaptor.createWithPayload(i8)
                    self._adaptor.addChild(root_0, i8_tree)

                    DOWN15=self.match(self.input, 2, self.FOLLOW_2_in_sig_primary872)

                    DOWN15_tree = self._adaptor.createWithPayload(DOWN15)
                    self._adaptor.addChild(root_0, DOWN15_tree)

                    #action start
                    retval.exp = self.check_updown(tab_symb, i8.text, 2)
                    #action end


                elif alt7 == 7:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:225:11: id2= IDENT
                    pass
                    root_0 = self._adaptor.nil()

                    id2=self.match(self.input, IDENT, self.FOLLOW_IDENT_in_sig_primary915)

                    id2_tree = self._adaptor.createWithPayload(id2)
                    self._adaptor.addChild(root_0, id2_tree)

                    #action start
                    retval.exp = self.check_ident(tab_symb, self.state_only, self.catch_free_clocks, self.deep, self.message, id2.text)
                    #action end


                elif alt7 == 8:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:228:11: PG expse= sig_expression1[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    PG16=self.match(self.input, PG, self.FOLLOW_PG_in_sig_primary956)

                    PG16_tree = self._adaptor.createWithPayload(PG16)
                    self._adaptor.addChild(root_0, PG16_tree)

                    self._state.following.append(self.FOLLOW_sig_expression1_in_sig_primary960)
                    expse = self.sig_expression1(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, expse.tree)
                    PD17=self.match(self.input, PD, self.FOLLOW_PD_in_sig_primary963)

                    PD17_tree = self._adaptor.createWithPayload(PD17)
                    self._adaptor.addChild(root_0, PD17_tree)

                    #action start
                    retval.exp = ((expse is not None) and [expse.exp] or [None])[0]
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_primary"

    class sig_constant_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_constant"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:232:1: sig_constant returns [exp] : ( T | F );
    def sig_constant(self, ):

        retval = self.sig_constant_return()
        retval.start = self.input.LT(1)

        root_0 = None

        T18 = None
        F19 = None

        T18_tree = None
        F19_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:233:10: ( T | F )
                alt8 = 2
                LA8_0 = self.input.LA(1)

                if (LA8_0 == T) :
                    alt8 = 1
                elif (LA8_0 == F) :
                    alt8 = 2
                else:
                    nvae = NoViableAltException("", 8, 0, self.input)

                    raise nvae

                if alt8 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:233:12: T
                    pass
                    root_0 = self._adaptor.nil()

                    T18=self.match(self.input, T, self.FOLLOW_T_in_sig_constant1008)

                    T18_tree = self._adaptor.createWithPayload(T18)
                    self._adaptor.addChild(root_0, T18_tree)

                    #action start
                    retval.exp = SigConstExpr(True)
                    #action end


                elif alt8 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:234:12: F
                    pass
                    root_0 = self._adaptor.nil()

                    F19=self.match(self.input, F, self.FOLLOW_F_in_sig_constant1023)

                    F19_tree = self._adaptor.createWithPayload(F19)
                    self._adaptor.addChild(root_0, F19_tree)

                    #action start
                    retval.exp = SigConstExpr(False)
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_constant"

    class sig_constraint_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.exp = None
            self.tree = None




    # $ANTLR start "sig_constraint"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:237:1: sig_constraint[tab_symb] returns [exp] : ( SYNC PG el= exp_list[tab_symb] PD | EXC PG el= exp_list[tab_symb] PD | INC PG e3= sig_expression1[tab_symb] COM e4= sig_exp[tab_symb] PD );
    def sig_constraint(self, tab_symb):

        retval = self.sig_constraint_return()
        retval.start = self.input.LT(1)

        root_0 = None

        SYNC20 = None
        PG21 = None
        PD22 = None
        EXC23 = None
        PG24 = None
        PD25 = None
        INC26 = None
        PG27 = None
        COM28 = None
        PD29 = None
        el = None

        e3 = None

        e4 = None


        SYNC20_tree = None
        PG21_tree = None
        PD22_tree = None
        EXC23_tree = None
        PG24_tree = None
        PD25_tree = None
        INC26_tree = None
        PG27_tree = None
        COM28_tree = None
        PD29_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:238:10: ( SYNC PG el= exp_list[tab_symb] PD | EXC PG el= exp_list[tab_symb] PD | INC PG e3= sig_expression1[tab_symb] COM e4= sig_exp[tab_symb] PD )
                alt9 = 3
                LA9 = self.input.LA(1)
                if LA9 == SYNC:
                    alt9 = 1
                elif LA9 == EXC:
                    alt9 = 2
                elif LA9 == INC:
                    alt9 = 3
                else:
                    nvae = NoViableAltException("", 9, 0, self.input)

                    raise nvae

                if alt9 == 1:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:238:12: SYNC PG el= exp_list[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    SYNC20=self.match(self.input, SYNC, self.FOLLOW_SYNC_in_sig_constraint1075)

                    SYNC20_tree = self._adaptor.createWithPayload(SYNC20)
                    root_0 = self._adaptor.becomeRoot(SYNC20_tree, root_0)

                    PG21=self.match(self.input, PG, self.FOLLOW_PG_in_sig_constraint1078)
                    self._state.following.append(self.FOLLOW_exp_list_in_sig_constraint1083)
                    el = self.exp_list(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, el.tree)
                    PD22=self.match(self.input, PD, self.FOLLOW_PD_in_sig_constraint1087)
                    #action start
                    retval.exp = self.check_sync(((el is not None) and [el.expl] or [None])[0])
                    #action end


                elif alt9 == 2:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:241:12: EXC PG el= exp_list[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    EXC23=self.match(self.input, EXC, self.FOLLOW_EXC_in_sig_constraint1138)

                    EXC23_tree = self._adaptor.createWithPayload(EXC23)
                    root_0 = self._adaptor.becomeRoot(EXC23_tree, root_0)

                    PG24=self.match(self.input, PG, self.FOLLOW_PG_in_sig_constraint1142)
                    self._state.following.append(self.FOLLOW_exp_list_in_sig_constraint1149)
                    el = self.exp_list(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, el.tree)
                    PD25=self.match(self.input, PD, self.FOLLOW_PD_in_sig_constraint1152)
                    #action start
                    retval.exp = self.check_exclus(((el is not None) and [el.expl] or [None])[0])
                    #action end


                elif alt9 == 3:
                    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:244:12: INC PG e3= sig_expression1[tab_symb] COM e4= sig_exp[tab_symb] PD
                    pass
                    root_0 = self._adaptor.nil()

                    INC26=self.match(self.input, INC, self.FOLLOW_INC_in_sig_constraint1181)

                    INC26_tree = self._adaptor.createWithPayload(INC26)
                    root_0 = self._adaptor.becomeRoot(INC26_tree, root_0)

                    PG27=self.match(self.input, PG, self.FOLLOW_PG_in_sig_constraint1184)
                    self._state.following.append(self.FOLLOW_sig_expression1_in_sig_constraint1189)
                    e3 = self.sig_expression1(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, e3.tree)
                    COM28=self.match(self.input, COM, self.FOLLOW_COM_in_sig_constraint1192)
                    self._state.following.append(self.FOLLOW_sig_exp_in_sig_constraint1197)
                    e4 = self.sig_exp(tab_symb)

                    self._state.following.pop()
                    self._adaptor.addChild(root_0, e4.tree)
                    PD29=self.match(self.input, PD, self.FOLLOW_PD_in_sig_constraint1200)
                    #action start
                    retval.exp = self.check_included(((e3 is not None) and [e3.exp] or [None])[0], ((e4 is not None) and [e4.exp] or [None])[0])
                    #action end


                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "sig_constraint"

    class exp_list_return(ParserRuleReturnScope):
        def __init__(self):
            ParserRuleReturnScope.__init__(self)

            self.expl = None
            self.tree = None




    # $ANTLR start "exp_list"
    # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:248:1: exp_list[tab_symb] returns [expl] : exp1= sig_expression1[tab_symb] ( COM exp2= sig_expression1[tab_symb] )* ;
    def exp_list(self, tab_symb):

        retval = self.exp_list_return()
        retval.start = self.input.LT(1)

        root_0 = None

        COM30 = None
        exp1 = None

        exp2 = None


        COM30_tree = None

        try:
            try:
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:249:10: (exp1= sig_expression1[tab_symb] ( COM exp2= sig_expression1[tab_symb] )* )
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:249:12: exp1= sig_expression1[tab_symb] ( COM exp2= sig_expression1[tab_symb] )*
                pass
                root_0 = self._adaptor.nil()

                self._state.following.append(self.FOLLOW_sig_expression1_in_exp_list1257)
                exp1 = self.sig_expression1(tab_symb)

                self._state.following.pop()
                self._adaptor.addChild(root_0, exp1.tree)
                #action start
                retval.expl = [((exp1 is not None) and [exp1.exp] or [None])[0]]
                #action end
                # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:250:13: ( COM exp2= sig_expression1[tab_symb] )*
                while True: #loop10
                    alt10 = 2
                    LA10_0 = self.input.LA(1)

                    if (LA10_0 == COM) :
                        alt10 = 1


                    if alt10 == 1:
                        # /home/gandrieu/cadbiomworkspace/Cadbiom_des_new/models/biosignal/translators/sigexpr_compiler.g:250:14: COM exp2= sig_expression1[tab_symb]
                        pass
                        COM30=self.match(self.input, COM, self.FOLLOW_COM_in_exp_list1275)

                        COM30_tree = self._adaptor.createWithPayload(COM30)
                        self._adaptor.addChild(root_0, COM30_tree)

                        self._state.following.append(self.FOLLOW_sig_expression1_in_exp_list1279)
                        exp2 = self.sig_expression1(tab_symb)

                        self._state.following.pop()
                        self._adaptor.addChild(root_0, exp2.tree)
                        #action start
                        retval.expl.append(((exp2 is not None) and [exp2.exp] or [None])[0])
                        #action end


                    else:
                        break #loop10





                retval.stop = self.input.LT(-1)


                retval.tree = self._adaptor.rulePostProcessing(root_0)
                self._adaptor.setTokenBoundaries(retval.tree, retval.start, retval.stop)


            except RecognitionException, re:
                self.reportError(re)
                self.recover(self.input, re)
                retval.tree = self._adaptor.errorNode(self.input, retval.start, self.input.LT(-1), re)
        finally:

            pass

        return retval

    # $ANTLR end "exp_list"


    # Delegated rules


    # lookup tables for DFA #1

    DFA1_eot = DFA.unpack(
        u"\14\uffff"
        )

    DFA1_eof = DFA.unpack(
        u"\14\uffff"
        )

    DFA1_min = DFA.unpack(
        u"\1\7\13\uffff"
        )

    DFA1_max = DFA.unpack(
        u"\1\42\13\uffff"
        )

    DFA1_accept = DFA.unpack(
        u"\1\uffff\1\1\7\uffff\1\2\2\uffff"
        )

    DFA1_special = DFA.unpack(
        u"\14\uffff"
        )


    DFA1_transition = [
        DFA.unpack(u"\2\1\3\11\3\uffff\3\1\4\uffff\1\1\1\uffff\1\1\11\uffff"
        u"\1\1"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"")
    ]

    # class definition for DFA #1

    DFA1 = DFA
    # lookup tables for DFA #3

    DFA3_eot = DFA.unpack(
        u"\13\uffff"
        )

    DFA3_eof = DFA.unpack(
        u"\13\uffff"
        )

    DFA3_min = DFA.unpack(
        u"\1\7\12\uffff"
        )

    DFA3_max = DFA.unpack(
        u"\1\42\12\uffff"
        )

    DFA3_accept = DFA.unpack(
        u"\1\uffff\1\1\6\uffff\1\2\2\uffff"
        )

    DFA3_special = DFA.unpack(
        u"\13\uffff"
        )


    DFA3_transition = [
        DFA.unpack(u"\2\1\6\uffff\3\1\4\uffff\1\1\3\10\10\uffff\1\1"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"")
    ]

    # class definition for DFA #3

    DFA3 = DFA
    # lookup tables for DFA #7

    DFA7_eot = DFA.unpack(
        u"\21\uffff"
        )

    DFA7_eof = DFA.unpack(
        u"\21\uffff"
        )

    DFA7_min = DFA.unpack(
        u"\1\7\5\uffff\1\2\12\uffff"
        )

    DFA7_max = DFA.unpack(
        u"\1\42\5\uffff\1\31\12\uffff"
        )

    DFA7_accept = DFA.unpack(
        u"\1\uffff\1\1\1\2\1\uffff\1\3\1\4\1\uffff\1\10\1\5\1\6\1\7\6\uffff"
        )

    DFA7_special = DFA.unpack(
        u"\21\uffff"
        )


    DFA7_transition = [
        DFA.unpack(u"\1\5\1\4\6\uffff\1\1\2\2\4\uffff\1\7\13\uffff\1\6"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"\1\11\1\10\2\uffff\2\12\5\uffff\2\12\10\uffff\3\12"),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u""),
        DFA.unpack(u"")
    ]

    # class definition for DFA #7

    DFA7 = DFA


    FOLLOW_sig_expression1_in_sig_expression148 = frozenset([24])
    FOLLOW_DOL_in_sig_expression151 = frozenset([1])
    FOLLOW_sig_constraint_in_sig_expression181 = frozenset([24])
    FOLLOW_DOL_in_sig_expression184 = frozenset([1])
    FOLLOW_sig_exp_in_sig_expression1230 = frozenset([1, 6])
    FOLLOW_DEFAULT_in_sig_expression1247 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_exp_in_sig_expression1252 = frozenset([1, 6])
    FOLLOW_sig_bool_in_sig_exp338 = frozenset([1, 7])
    FOLLOW_WHEN_in_sig_exp355 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_bool_in_sig_exp389 = frozenset([1, 7])
    FOLLOW_sig_bool_and_in_sig_bool471 = frozenset([1, 14])
    FOLLOW_OR_in_sig_bool487 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_bool_and_in_sig_bool492 = frozenset([1, 14])
    FOLLOW_sig_primary_in_sig_bool_and561 = frozenset([1, 13])
    FOLLOW_AND_in_sig_bool_and577 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_primary_in_sig_bool_and582 = frozenset([1, 13])
    FOLLOW_NOT_in_sig_primary656 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_primary_in_sig_primary661 = frozenset([1])
    FOLLOW_sig_constant_in_sig_primary704 = frozenset([1])
    FOLLOW_EVENT_in_sig_primary751 = frozenset([22])
    FOLLOW_PG_in_sig_primary753 = frozenset([7, 8, 15, 16, 17, 22, 23, 34])
    FOLLOW_sig_expression1_in_sig_primary758 = frozenset([23])
    FOLLOW_PD_in_sig_primary761 = frozenset([1])
    FOLLOW_WHEN_in_sig_primary798 = frozenset([22])
    FOLLOW_PG_in_sig_primary800 = frozenset([7, 8, 15, 16, 17, 22, 23, 34])
    FOLLOW_sig_expression1_in_sig_primary805 = frozenset([23])
    FOLLOW_PD_in_sig_primary808 = frozenset([1])
    FOLLOW_IDENT_in_sig_primary838 = frozenset([3])
    FOLLOW_3_in_sig_primary840 = frozenset([1])
    FOLLOW_IDENT_in_sig_primary870 = frozenset([2])
    FOLLOW_2_in_sig_primary872 = frozenset([1])
    FOLLOW_IDENT_in_sig_primary915 = frozenset([1])
    FOLLOW_PG_in_sig_primary956 = frozenset([7, 8, 15, 16, 17, 22, 23, 34])
    FOLLOW_sig_expression1_in_sig_primary960 = frozenset([23])
    FOLLOW_PD_in_sig_primary963 = frozenset([1])
    FOLLOW_T_in_sig_constant1008 = frozenset([1])
    FOLLOW_F_in_sig_constant1023 = frozenset([1])
    FOLLOW_SYNC_in_sig_constraint1075 = frozenset([22])
    FOLLOW_PG_in_sig_constraint1078 = frozenset([7, 8, 15, 16, 17, 22, 25, 34])
    FOLLOW_exp_list_in_sig_constraint1083 = frozenset([23])
    FOLLOW_PD_in_sig_constraint1087 = frozenset([1])
    FOLLOW_EXC_in_sig_constraint1138 = frozenset([22])
    FOLLOW_PG_in_sig_constraint1142 = frozenset([7, 8, 15, 16, 17, 22, 25, 34])
    FOLLOW_exp_list_in_sig_constraint1149 = frozenset([23])
    FOLLOW_PD_in_sig_constraint1152 = frozenset([1])
    FOLLOW_INC_in_sig_constraint1181 = frozenset([22])
    FOLLOW_PG_in_sig_constraint1184 = frozenset([7, 8, 15, 16, 17, 22, 25, 34])
    FOLLOW_sig_expression1_in_sig_constraint1189 = frozenset([25])
    FOLLOW_COM_in_sig_constraint1192 = frozenset([7, 8, 15, 16, 17, 22, 34])
    FOLLOW_sig_exp_in_sig_constraint1197 = frozenset([23])
    FOLLOW_PD_in_sig_constraint1200 = frozenset([1])
    FOLLOW_sig_expression1_in_exp_list1257 = frozenset([1, 25])
    FOLLOW_COM_in_exp_list1275 = frozenset([7, 8, 15, 16, 17, 22, 25, 34])
    FOLLOW_sig_expression1_in_exp_list1279 = frozenset([1, 25])



def main(argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    from antlr3.main import ParserMain
    main = ParserMain("sigexpr_compilerLexer", sigexpr_compiler)
    main.stdin = stdin
    main.stdout = stdout
    main.stderr = stderr
    main.execute(argv)


if __name__ == '__main__':
    main(sys.argv)



## Filename    : MCLAnalyser.py
## Author(s)   : Michel Le Borgne
## Created     : 03/2012
## Revision    :
## Source      :
##
## Copyright 2012 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux - IRISA/IRSET
##
"""
Main class to perform dynamic analysis of Cadbiom chart models.
"""
from cadbiom.antlr3 import ANTLRFileStream, CommonTokenStream

from cadbiom.models.guard_transitions.translators.chart_xml import \
        MakeModelFromXmlFile

from cadbiom.models.guard_transitions.translators.chart_xml_pid import \
        MakeModelFromPidFile

from cadbiom.models.guard_transitions.analyser.ana_visitors import TableVisitor
from cadbiom.models.clause_constraints.mcl.MCLTranslators import  GT2Clauses
from cadbiom.models.clause_constraints.mcl.CLUnfolder import CLUnfolder
from cadbiom.models.clause_constraints.mcl.MCLSolutions import \
        extract_dimacs_init_frontier, MCLException

from cadbiom.models.clause_constraints.CLDynSys import CLDynSys
from cadbiom.models.guard_transitions.translators.cadlangLexer import cadlangLexer
from cadbiom.models.guard_transitions.translators.cadlangParser import cadlangParser
from cadbiom.models.guard_transitions.chart_model import ChartModel
from cadbiom.models.clause_constraints.mcl.MCLSolutions import FrontierSolution
from cadbiom.models.clause_constraints.mcl.MCLQuery import MCLSimpleQuery

#import random
import sys
TRACEFILE = sys.stdout

class MCLAnalyser(object):
    """
    query a UCL dynamical system and analyses solutions
    """
    NB_SOL = 10  # for mac search - to be optimized??

    def __init__(self, report):
        """
        Constructor - the built analyser is void
        @param report: a standard reporter
        """
        self.cl_dyn_sys = None     # dynamical system in clause constraint form
        self.unfolder = None       # computation management: unfolding
        self.reporter = report     # for generic error display
        self.opti = True           # turn on optimizations for translation

    # building
    def build_from_chart_model(self, model):
        """
        Build an MCLAnalyser from a chartmodel object
        @param model: object - ChartModel
        """
        # reloading a MCLAnalyser is forbidden - create a new one
        if self.unfolder:
            raise MCLException("This MCLAnalyser is already initialized")
        # building CLsys
        vtab = TableVisitor(self.reporter)
        model.accept(vtab)
        if self.reporter.error:
            return
        clds = CLDynSys( vtab.tab_symb, self.reporter)
        if self.reporter.error:
            return
        comp_visitor = GT2Clauses(clds, self.reporter, self.opti)
        model.accept(comp_visitor)
        if self.reporter.error:
            return
        self.cl_dyn_sys = clds

        # build unfolder
        self.unfolder = CLUnfolder(clds)

    def build_from_chart_file(self, file_name):
        """
        Build an MCLAnalyser from a .bcx file
        @param file_name: str - path of the .bcx file
        """
        parsing = MakeModelFromXmlFile(file_name)
        # chart model
        cmodel = parsing.get_model()
        self.build_from_chart_model(cmodel)

    def build_from_pid_file(self,
                            file_name, has_clock=True, ai_interpretation=0):
        """
        Build an MCLAnalyser from a .xml file of PID database
        @param file_name: str - path of .xml file
        """
        parsing = MakeModelFromPidFile(file_name,
                                    self.reporter, has_clock, ai_interpretation)
        # chart model
        cmodel = parsing.get_model()
        self.build_from_chart_model(cmodel)

    def build_from_cadlang(self, file_name):
        """
        Build an MCLAnalyser from a .cal file of PID database
        @param file_name: str - path of .cal file
        """
        creporter = self.reporter
        fstream = ANTLRFileStream(file_name)
        lexer = cadlangLexer(fstream)
        lexer.set_error_reporter(creporter)
        parser = cadlangParser(CommonTokenStream(lexer))
        parser.set_error_reporter(creporter)
        model = ChartModel(file_name)
        parser.cad_model(model)
        # chart model
        cmodel = parser.model
        self.build_from_chart_model(cmodel)



    # solution processing
    def less_activated_solution(self, dimacs_solution_list):
        """
        return the solution with the less activated  states
        @param dimacs_solution_list: either a DimacsFrontierSol or DimacsActFrontSol list
        @return: a solution object of the type in the list
        """
        if not dimacs_solution_list:
            self.reporter.display(
                    "Trying to find a smallest solution without solution list")
            return None

        smallest_sol = dimacs_solution_list[0]
        la_number = smallest_sol.nb_activated()

        for sol in dimacs_solution_list[1:]:
            if sol.nb_activated() < la_number:
                smallest_sol = sol
                la_number = sol.nb_activated()
        return smallest_sol

    def __prune_frontier_solution(self, fsol, query, max_step):
        """
        take a frontier condition  which induces property prop and reduce the
        number of activated variables from this solution while inducing prop,
        @param fsol: a DimacsFrontierSol
        @param query: MCLQuery
        @return: a DimacsfrontierSol

        ASSERT: fsol is a frontier condition implying sq satisfiability
        """

#        # for debug - too expansive to be activated anytime
#        start = []
#        for s in fsol.frontier_values:
#            start.append([s])
#        assert(self.sq_is_satisfiable(query, max_step))
        to_prune = fsol
        current_len = fsol.nb_activated() # number of act. places in solution
        next_len = 0
        is_pruned = True

        while is_pruned:
            # find at most 10 ic frontier solutions with same no activated
            # frontier place - return current_small if no more sol
            dimacs_solutions = self.__solve_with_inact_fsolution(
                                                                 to_prune,
                                                                 query,
                                                                 max_step)
            if len(dimacs_solutions) == 1:
                return dimacs_solutions[0]  # found no new solutions
            # seach for less activated solutions
            next_small = self.less_activated_solution(dimacs_solutions)
            next_len = next_small.nb_activated()
            is_pruned = current_len > next_len
            if is_pruned:
                to_prune = next_small
                current_len = next_len
        # never reach this point
        raise MCLException("_prune_frontier_solution: what happened??")

    # for unit test
    def test_prune_frontier_solution(self, fsol, query, max_step):
        """
        For tests only
        """
        return self.__prune_frontier_solution(fsol, query, max_step)


    # generic solver for simple queries
    def sq_is_satisfiable(self, query, max_step):
        """
        @param query: MCLQuery
        @param max_step: int - length of unfolding

        @return: boolean

        @warning: no_frontier places are initialized to False in simple queries
        """
        # setting parameters
        self.unfolder.reset()
        self.unfolder.set_initial_property(query.start_prop)
        self.unfolder.set_dim_initial_property(query.dim_start)
        self.unfolder.set_current_property(query.inv_prop)
        self.unfolder.set_dim_current_property(query.dim_inv)
        self.unfolder.set_final_property(query.final_prop)
        self.unfolder.set_dim_final_property(query.dim_final)
        self.unfolder.set_variant_property(query.variant_prop)
        self.unfolder.set_dim_variant_property(query.dim_variant_prop)

        self.unfolder.set_minimal_steps(query.steps_before_reach)
        # go
        return self.unfolder.squery_is_satisfied(max_step)


    def sq_solutions(self, query, max_step, max_sol, vvars):
        """
        @param query: MCLQuery
        @param max_step: int - length of unfolding
        @param vvars: variables for which solutions must differ:
                    list<int> Dimacs code
        @return: a list of RawSolution objects

        @warning:  no_frontier places are initialized to False in simple queries
            except if initial condition.

        @warning: if the query includes variant constraints, the horizon (max_step) is
            automatically adjusted to min(max_step, len(variant_constraints)).
        """
        # setting parameters
        self.unfolder.reset()
        self.unfolder.set_initial_property(query.start_prop)
        self.unfolder.set_dim_initial_property(query.dim_start)
        self.unfolder.set_current_property(query.inv_prop)
        self.unfolder.set_dim_current_property(query.dim_inv)
        self.unfolder.set_final_property(query.final_prop)
        self.unfolder.set_dim_final_property(query.dim_final)
        self.unfolder.set_variant_property(query.variant_prop)
        self.unfolder.set_dim_variant_property(query.dim_variant_prop)

        self.unfolder.set_minimal_steps(query.steps_before_reach)
        # go
        return self.unfolder.squery_solve(vvars, max_step, max_sol)


    def sq_frontier_solutions(self, query, max_step, max_sol):
        """
        Compute active frontier places and timing
        @param query: MCLQuery
        @param max_step: int - length of unfolding
        @param max_sol: max number of solutions
        @return:  list<FrontierSolution>  set of frontier place names which
                                      must be activated for implying query satisfaction
        @warning:  no_frontier places are initialized to False

        """
        vvars = self.unfolder.get_frontier()
        l_sol = self.sq_solutions(query, max_step, max_sol, vvars)
        out = []
        for sol in l_sol:
            fsol = FrontierSolution.from_raw(sol)
            out.append(fsol)
        return out

    def __sq_dimacs_frontier_solutions(self, query, max_step, max_sol):
        """
        @param query: MCLQuery
        @param max_step: int - length of unfolding
        @param max_sol:int - max number of solutions for each solve
        @return:  list<DimacsFrontierSol>

        """
        frontier = self.unfolder.get_frontier()
        lsol = self.sq_solutions(query, max_step, max_sol, frontier)
        return extract_dimacs_init_frontier(lsol)


    # only for unit test
    def test_dimacs_frontier_sol(self, query, max_step, max_sol):
        """ for test only """
        return self.__sq_dimacs_frontier_solutions(query, max_step, max_sol)



    def __solve_with_inact_fsolution(self, fsolution, query, max_step):
        """
        frontier states not activated in  fsolution (frontier solution) forced
        to be False at start solve and return frontier solution list with this constraint
        @param fsolution: list<DimacsFrontierSol> -  one solution to obtain the property in dimacs
        @param query: current query
        @param max_step: int - horizon

        @precondition: fsolution is a frontier solution for the query
        """

        # collect inactivated places
        final_dsol = \
            [[var] for var in fsolution.frontier_values if var < 0]
        if query.dim_start:
            query.set_dim_start(final_dsol + query.dim_start)
        else:
            query.set_dim_start(final_dsol)
        # must return fsol if no more solution
        return self.__sq_dimacs_frontier_solutions(query, max_step,
                                                   MCLAnalyser.NB_SOL)


    # only for unit test
    def test_solve_with_inact_fsolution(self, fsolution, query, max_step):
        """
        unit test accessible
        """
        self.__solve_with_inact_fsolution(fsolution, query, max_step)



    def __mac_exhaustive_search(self, query, nb_step):
        """
        @param query: MCLQuery
        @param nb_step: int - number of dynamical step
        @return list<DimacsFrontierSol>
        """
        # list of timed minimal activation conditions on frontier (dimacs code)
        # i.e list<DimacsFrontierSol>
        mac_list = []

        reachable = self.sq_is_satisfiable(query, nb_step)
        # get the minimum number of steps for reachability
        min_step = self.unfolder.get_current_step() - 1
        query.set_steps_before_reach(min_step)

        while reachable :
            # compute forbidden solutions: already discovered macs
            forbidden_sol = []
            for dsol in mac_list:
                dimacs_clause = []
                for var in dsol.frontier_values:
                    if var > 0:
                        dimacs_clause.append(-var)       # OR( not var)
                forbidden_sol.append(dimacs_clause)

            # reachable - solutions differ on frontier: 2 different solutions
            # to avoid a all activated solution - lfsol:list<DimacsFrontierSol>
            query.set_dim_start(forbidden_sol)
            lfsol = self.__sq_dimacs_frontier_solutions(query, nb_step, 2)
            if len(lfsol)>0:
                small_sol = self.less_activated_solution(lfsol)
                start = []
                for ssol in small_sol.frontier_values:
                    start.append([ssol])
                #assert(self.sq_is_satisfiable(query, nb_step))
            else:
                reachable = False
                break

            current_mac = self.__prune_frontier_solution(small_sol,
                                                         query, nb_step)
            if current_mac.nb_activated() == 0:
                reachable = False
            else :
                mac_list.append(current_mac)
        return mac_list



    def next_mac(self, query, nb_step):
        """
        if st_prop contains negation of mac conditions,
        return a different mac if any
        same parameters as __mac_exhaustive_search
        Used for search on cluster

        @param query: MCLQuery
        @param nb_step: int - number of dynamical step
        @return: a list of variables (list<string>)
        """
        lfsol = self.__sq_dimacs_frontier_solutions(query, nb_step, 2)
        if len(lfsol)>0:
            small_fsol = self.less_activated_solution(lfsol)
        else:
            return None

        current_mac = self.__prune_frontier_solution(small_fsol,
                                                     query,
                                                     nb_step)
        vmac = FrontierSolution.from_dimacs_front_sol(current_mac)
        return vmac

    def mac_search(self, query, nb_step):
        """
        @param query: MCLQuery
        @param nb_step: int - number of dynamical step
        @return: list<FrontierSolution>
        """
        mac_list = self.__mac_exhaustive_search(query, nb_step)
        # convert to FrontierSolution list
        vmac_list = []
        for mac in mac_list:
            vmac = FrontierSolution.from_dimacs_front_sol(mac)
            vmac_list.append(vmac)
        return vmac_list

    def is_strong_activator(self, act_sol, query):
        """
        test if an activation condition is a strong one (independent of timing)
        @param act_sol: FrontierSolution with activation condition
        @param query: the query used for act_sol condition
        """
        nb_step = len(act_sol.ic_sequence) - 1
        query_1 = MCLSimpleQuery.from_frontier_sol_same_timing(act_sol,
                                                              self.unfolder)
        inv_prop = 'not('+query.final_prop+')'
        query_2 = MCLSimpleQuery(None, inv_prop, None)
        query_1 = query_2.merge(query_1)
        return not(self.sq_is_satisfiable(query_1, nb_step))

    def next_inhibitor(self, mac, query):
        """
        if st_prop contains negation of mac conditions,
        return a different mac if any
        same parameters as __mac_exhaustive_search
        Used for search on cluster

        @param mac: FrontierSolution
        @param query: MCLQuery
        @param nb_step: int - number of dynamical step
        @return: a list of variables (list<string>)
        """
        # query with mac init frontier places and  timing
        inh_query1 = MCLSimpleQuery.from_frontier_sol_same_timing(mac,
                                                                  self.unfolder)
        # query with negation of final prop as invariant property
        if not query.final_prop:
            raise MCLException("Query must have a final property")
        if not query.inv_prop:
            inv_prop = "not (" + query.final_prop + ")"
        else:
            inv_prop = query.inv_prop + "and (not (" + query.final_prop +"))"
        inh_query2 = MCLSimpleQuery(None, inv_prop, None)
        # init + timing + final property not reachable
        inh_query = inh_query1.merge(inh_query2)
        nb_step = len(inh_query.variant_prop)
        assert(nb_step == len(mac.ic_sequence))

        # search solutions - diseable aux clauses
        self.unfolder.set_include_aux_clauses(False)
        next_inhib = self.next_mac(inh_query, nb_step)
        self.unfolder.set_include_aux_clauses(True)
        return next_inhib


    def mac_inhibitor_search(self, mac, query, max_sol):
        """
        Search inhibitors for a mac scenario

        @param mac: the mac
        @param query: the property enforced by the mac
        @param max_sol: limit on the number of solutions
        @param max_sol: maximum number of solutions

        @return: a list of frontier solution
        """
        # query with mac init frontier places and  timing
        inh_query1 = MCLSimpleQuery.from_frontier_sol(mac)
        # query with negation of final prop as invariant property
        if not query.final_prop:
            raise MCLException("Query must have a final property")
        if not query.inv_prop:
            inv_prop = "not (" + query.final_prop + ")"
        else:
            inv_prop = query.inv_prop + "and (not (" + query.final_prop +"))"
        inh_query2 = MCLSimpleQuery(None, inv_prop, None)
        # init + timing + final property not reachable
        inh_query = inh_query1.merge(inh_query2)

        nb_step = len(inh_query.variant_prop)
        assert(nb_step == len(mac.ic_sequence))
        # search solutions - diseable aux clauses
        self.unfolder.set_include_aux_clauses(False)
#        lsol = self.sq_frontier_solutions(inh_query, nb_step, max_sol)
        lsol = self.mac_search(inh_query, nb_step)
        self.unfolder.set_include_aux_clauses(True)
        return lsol

    def is_strong_inhibitor(self, in_sol, query):
        """
        test if an activation condition inhibitor is a trong one
        (i.e independent of timing)
        @param in_sol: FrontierSolution with activation condition and inhibition
        @param query: the property which is inhibed
        """
        nb_step = len(in_sol.ic_sequence) - 1
        query_1 = MCLSimpleQuery.from_frontier_sol_new_timing(in_sol,
                                                              self.unfolder)
        # negation of the query
        if not query.final_prop:
            raise MCLException("Query must have a final property")
        if not query.inv_prop:
            inv_prop = "not (" + query.final_prop + ")"
        else:
            inv_prop = query.inv_prop + "and (not (" + query.final_prop +"))"
        inh_query = MCLSimpleQuery(None, inv_prop, None)
        query_1 = inh_query.merge(query_1)
        return not self.sq_is_satisfiable(query_1, nb_step)


# helper functions
def only_in_l1(ll1, ll2):
    """
    @param ll1: list<any> - list of element
    @param ll2: list<any> - list of element
    @return: only_l1 list<any> - list of element that are only in the first list (l1\l2)
    """
    only_l1 = []
    for el1 in ll1:
        if el1 not in ll2:
            only_l1.append(el1)
    only_l1.sort()
    return only_l1




## Filename    : TestExprCompiler.py
## Author(s)   : Michel Le Borgne
## Created     : 04/2010
## Revision    :
## Source      :
##
## Copyright 2010 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and IRISA has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE
##
##
## Contributor(s): Geoffroy Andrieux, Nolwenn Le Meur
##
"""
Tests for the biosignal expression compiler
"""

import unittest
from cadbiom.models.biosignal.sig_expr import SigDefaultExpr, SigIdentExpr, \
                SigConstExpr, SigDiffExpr, SigEqualExpr, SigNotExpr, \
                SigEventExpr, SigWhenExpr, SigSyncBinExpr
from cadbiom.models.biosignal.translators.sigexpr_compiler import sigexpr_compiler
from cadbiom.models.biosignal.translators.sigexpr_lexer import sigexpr_lexer
from cadbiom.antlr3 import ANTLRStringStream, CommonTokenStream, RecognitionException


class Reporter(object):
    """
    Simple reporter
    """
    def __init__(self):
        self.mem = ">>>>>>>>"
        self.error = False
        pass

    def display(self, mess):
        """
        Just register messages
        """
        self.error = True
        print '--->' + mess
        self.mem = self.mem + "\n" + mess



class TestExpressionAnalyzer(unittest.TestCase):
    """
    The setup generates a collection of biosignal  expressions
    Test all elementary operators
    """

    def setUp(self):
        self.exp1 = SigIdentExpr('X1')
        self.exp2 = SigIdentExpr('X2')
        self.exp3 = SigIdentExpr('X3')
        self.exp4 = SigIdentExpr('X4')
        self.exp5 = SigConstExpr(True)
        self.exp6 = SigConstExpr(False)

        self.exp10 = SigDefaultExpr(self.exp1, self.exp2)    # X1 default X2
        self.exp11 = SigWhenExpr(self.exp3, self.exp4)       # X3 when X4
        self.exp12 = SigDiffExpr(self.exp1, self.exp4)       # X1 != X4
        self.exp13 = SigEqualExpr(self.exp2, self.exp4)      # X2 == X4
        self.exp14 = SigNotExpr(self.exp1)          # not X1
        self.exp15 = SigEventExpr(self.exp10)       # event (X1 default X2)
        self.exp16 = SigNotExpr(self.exp10)         # not (X1 default X2)
#        #(X1 != X4) + not (X1 default X2)
#        self.exp17 = SigPolyBinExpr('+', self.exp12, self.exp16)
#        # (not X1)*(not (X1 default X2))
#        self.exp18 = SigPolyBinExpr('*', self.exp14, self.exp16)
#        # (not X1)-(not (X1 default X2))
#        self.exp19 = SigPolyBinExpr('-', self.exp14, self.exp16)
#        # ((not X1)-(not (X1 default X2)))^2
#        self.exp20 = SigPolyPowExpr(self.exp19, 2)
#        self.exp21 = SigPolyPowExpr(self.exp19, 1)
#        # (event (X1 default X2))^2
#        self.exp22 = SigPolyPowExpr(self.exp15, 2)

        self.st1 = SigIdentExpr('S1')
        self.st2 = SigIdentExpr('S2')
        self.st3 = SigIdentExpr('S3')

        self.err_rep = Reporter()
        self.tab = {}
        self.tab['X1'] = "emit", 1
        self.tab['X2'] = "input", 1
        self.tab['X3'] = "emit", 1
        self.tab['X4'] = "emit", 1
        self.tab['S1'] = "state", 1
        self.tab['S2'] = "state", 1
        self.tab['S3'] = "state", 1

        self.err_rep = Reporter()

    def tearDown(self):
        """
        Nothing to do
        """
        pass

    def test_equal(self):
        """
        test expression equality (necessary for tests!!)
        """
        # ident
        res = self.exp1.test_equal(self.exp1)
        self.assert_(res, "Error in Expression equality Test01")

        res = self.exp1.test_equal(self.exp2)
        self.assert_(not res, "Error in Expression equality Test02")
        # default
        exp10 = SigDefaultExpr(self.exp1, self.exp2) # X1 default X2
        exp101 = SigDefaultExpr(exp10, self.exp3) #( X1 default X2) default X3
        res = exp101.test_equal(exp101)
        self.assert_(res, "Error in Expression equality Test1")
        # when
        exp11 = SigWhenExpr(self.exp3, self.exp4) # X3 when X4
        exp111 = SigWhenExpr(exp11, self.exp2) # (X3 when X4) when X2
        res = exp111.test_equal(exp111)
        self.assert_(res, "Error in Expression equality Test2")

        res = exp111.test_equal(exp101)
        self.assert_(not res, "Error in Expression equality Test3")
        # when and default
        exp112 = SigDefaultExpr(self.exp1, exp11)  # X1 default (X3 when X4)
        res = exp112.test_equal(exp112)
        self.assert_(res, "Error in Expression equality Test4")

        res = exp112.test_equal(exp101)
        self.assert_(not res, "Error in Expression equality Test5")

        # binary bool exp
        exp12 = SigSyncBinExpr('or', self.exp1, self.exp2)
        exp121 = SigWhenExpr(self.exp3, exp12)
        res = exp121.test_equal(exp121)
        self.assert_(res, "Error in Expression equality Test6")

        exp13 = SigSyncBinExpr('and', self.exp1, self.exp2)
        exp131 = SigWhenExpr(self.exp3, exp13)
        res = exp131.test_equal(exp131)
        self.assert_(res, "Error in Expression equality Test7")

        res = exp121.test_equal(exp131)
        self.assert_(not res, "Error in Expression equality Test8")
        # binary bool exp whith constants
        exp14 = SigSyncBinExpr('or', self.exp1, SigConstExpr(True))
        exp141 = SigSyncBinExpr('or', self.exp1, SigConstExpr(False))
        res = exp141.test_equal(exp141)
        self.assert_(res, "Error in Expression equality Test9")

        res = exp141.test_equal(exp14)
        self.assert_(not res, "Error in Expression equality Test10")


    def test_default(self):
        """
        default left associativity
        """
        sig_str = "X1 default X2 default X3 $"
        reader = ANTLRStringStream(sig_str)
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for default")

        exp10 = SigDefaultExpr(self.exp1, self.exp2) # X1 default X2
        exp101 = SigDefaultExpr(exp10, self.exp3) #( X1 default X2) default X3
        res = exp101.test_equal(exp_obj.exp)
        self.assert_(res, "Error in Expression analyzer for default")

    def test_when(self):
        """
        when left associativity
        """
        reader = ANTLRStringStream("X3 when X4 when X2 $")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for when")
        exp11 = SigWhenExpr(self.exp3, self.exp4) # X3 when X4
        exp111 = SigWhenExpr(exp11, self.exp2) # (X3 when X4) when X2
        res = exp111.test_equal(exp_obj.exp)
        self.assert_(res, "Error in Expression analyzer for when")

    def test_default_when(self):
        """
        priority of when over default
        """
        reader = ANTLRStringStream("(X1 default X3 when X4)$")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for default_when")
        exp11 = SigWhenExpr(self.exp3, self.exp4) # X3 when X4
        exp112 = SigDefaultExpr(self.exp1, exp11)  # X1 default (X3 when X4)
        res = exp112.test_equal(exp_obj.exp)
        mess = "Error in Expression analyzer for X1 default X3 when X4"
        self.assert_(res, mess)

    def test_void(self):
        """
        void string : default value True
        """
        reader = ANTLRStringStream("  $")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for void")
        expv = SigConstExpr(True)
        res = expv.test_equal(exp_obj.exp)
        mess = "Error in void expression - must default to True"
        self.assert_(res, mess)


#        exp11 = SigWhenExpr(self.exp3,self.exp4)  # X3 when X4
#        exp112 = SigDefaultExpr(self.exp1, exp11) # X1 default (X3 when X4)
#        res = exp112.test_equal(exp_obj.exp)
#        mess = "Error in Expression analyzer for (X1 and X3 and X4)"
#        self.assert_(res, mess)

    def test_when_or(self):
        """
        boolean operator precedence on when operators
        """
        reader = ANTLRStringStream("X3 when X4 or X2 $")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for when")
        exp11 = SigSyncBinExpr('or', self.exp4, self.exp2) # X4 or X2
        exp111 = SigWhenExpr(self.exp3, exp11) # X3 when (X4 or X2)
        res = exp111.test_equal(exp_obj.exp)
        self.assert_(res, "Error in Expression analyzer for when_or")

    def test_type_error1(self):
        """
        boolean expr on states in default
        """
        reader = ANTLRStringStream("X3 default (S3 or S2) $")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        #parser.state_only = True
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for when")

        res = self.err_rep.error
        #print self.err_rep.mem
        self.assert_(res, "Type checking error in test_type_error1")

    def test_type_error2(self):
        """
        signal in when  with state_only option
        """
        reader = ANTLRStringStream("X3 when (S3 or X2) $")
        lexer = sigexpr_lexer(reader)
        parser = sigexpr_compiler(CommonTokenStream(lexer))
        parser.set_error_reporter(self.err_rep)
        parser.state_only = True
        try:
            exp_obj = parser.sig_expression(self.tab)
        except RecognitionException , exc:
            #parser.displayExceptionMessage(exc)
            self.assert_(True, "Compiler error for when")

        res = self.err_rep.error
        print self.err_rep.mem
        self.assert_(res, "Type checking error in test_type_error1")





## Filename    : CLDynSys.py
## Author(s)   : Michel Le Borgne
## Created     : 05/2011
## Revision    : 
## Source      : 
##
## Copyright 2011 : IRISA/IRSET
##
## This library is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published
## by the Free Software Foundation; either version 2.1 of the License, or
## any later version.
##
## This library is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF
## MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  The software and
## documentation provided here under is on an "as is" basis, and  has
## no obligations to provide maintenance, support, updates, enhancements
## or modifications.
## In no event shall IRISA be liable to any party for direct, indirect,
## special, incidental or consequential damages, including lost profits,
## arising out of the use of this software and its documentation, even if
## IRISA have been advised of the possibility of such damage.  See
## the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this library; if not, write to the Free Software Foundation,
## Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
##
## The original code contained here was initially developed by:
##
##     Michel Le Borgne.
##     IRISA
##     Symbiose team
##     IRISA  Campus de Beaulieu
##     35042 RENNES Cedex, FRANCE 
##     
##
## Contributor(s): Geoffroy Andrieux
##
"""
Classes for building clause constraint models
"""
class Clause():
    """
    Objects representing logical clauses
    a clause is represented as a list of literals.
    """
    def __init__(self, list_lit=[]):
        self.list_of_literals = list_lit
        
    def add_lit(self, lit):
        """
        add a literal to the clause
        @param lit: Literal object
        """
        self.list_of_literals.append(lit)
        
    def __str__(self):
        """
        To print
        """
        clause_string = '$'
        for lit in self.list_of_literals[:-1]:
            clause_string += lit.__str__()+', '
        clause_string = clause_string + self.list_of_literals[-1].__str__()+'$'
        return clause_string
        
    def __repr__(self):
        return self.__str__()
    
    def equal(self, cl2):
        """
        test if two clauses have the same literals
        """
        if len(self.list_of_literals) != len(cl2.list_of_literals):
            return  False       
        sc1 = sorted(self.list_of_literals, cmp=lit_cmp)
        sc2 = sorted(cl2.list_of_literals, cmp=lit_cmp)
        for i in range(len(sc1)):
            if not lit_cmp(sc1[i], sc2[i]) == 0:
                return False
        return True
    
# helper functions mainly for tests    
def clause_cmp(cl1, cl2):
    """
    lexicographic order on ordered list of literals
    """
    sc1 = sorted(cl1.list_of_literals, cmp=lit_cmp)
    sc2 = sorted(cl2.list_of_literals, cmp=lit_cmp) 
    len1 = len(sc1)
    len2 = len(sc2)
    rlen = min(len1, len2)
    for i in range(rlen):
        cmp = lit_cmp(sc1[i], sc2[i])
        # if not equal - clauses ordered as literals   
        if cmp != 0:
            return cmp
    # the shorter clause is a prefix of the longuest one
    if len1 < len2:
        return -1
    elif len2 < len1:
        return 1
    else:
        return 0
    
def clause_list_equal(lc1, lc2):
    """
    compare two clause lists
    """
    if len(lc1) != len(lc2):
        return False
    slc1 = sorted(lc1, cmp=clause_cmp)
    slc2 = sorted(lc2, cmp=clause_cmp)
    for i in range(len(slc1)):
        if not slc1[i].equal(slc2[i]):
            return False
    return True
    
    
class Literal():
    """
    Object representing literals.
    A literal is a pair (string, boolean). The string is the variable name 
    and the boolean the sign of the literal. 
    """
    
    def __init__(self, name, sign):
        self.name = name
        self.sign = sign
        
    def lit_not(self):
        """
        returns the negation of the literal
        """
        return Literal(self.name, not self.sign)
    
    def equal(self, lit):
        """
        literal equality
        """
        return (lit.name == self.name) and (lit.sign == self.sign) 
    
    def opposite(self, lit):
        """
        true if one literal is the negation of the other
        """
        return (lit.name == self.name) and (lit.sign ==  (not self.sign)) 
                 
    def __str__(self):
        """
        For printing
        """
        if self.sign:
            return self.name
        else:
            return 'not '+self.name
        
    def code_name(self):
        """
        returns a string representing the literal
        """
        if self.sign:
            return self.name
        else:
            return '_' + self.name
        
    def __repr__(self):
        return self.__str__()

def lit_cmp(lit1, lit2):
    """
    for sorting clauses
    @param lit1, lit2: Literal
    @return:  1 if l1>l2, 0 if equal, -1 if l1<l2
    """
    if lit1.equal(lit2):
        return 0
    if lit1.name == lit2.name:
        if lit2.sign:  # l1 > l2
            return 1
        else:
            return -1 # l1 < l2
    else:
        if lit1.name < lit2.name:
            return -1
        else:
            return 1
    
    
        
class CLDynSys(object):
    """
    This class describe a dynamic system in clause form.
    @param symb_tab: name (string) -> type
    @param report: reporter for error reporting
    
    The frontier, free_clock, place_clock list are used for extractiong informations
    from solutions. The place_cock list is used for generating the structural constraint
    OR(P and h) implying that at least one clock transition must be activated at each step.
    """
    
    def __init__(self, symb_tab, report):
        # symbol table of charter model produced by Table_visitor
        # completed by add_free_clock
        self.symb_tab = symb_tab             
        self.report = report           # reporter for errors
        # set of variables of the dynamic system (including aux vars)
        self.base_var_set = set()      
        self.list_clauses = []         # clause form of the dynamic
        # structural constraints valid if there is no timing constraints
        self.aux_list_clauses = []     
        self.frontier = []             # frontier of the model 
        # complement of the frontier in the set of variables
        self.no_frontier = []          
        
        self.free_clocks = []          # free clocks inputs
        # dictionary clock h -> [place P] where P is the place 
        # preceding free clock h 
        self.place_clocks = dict()    
        self.inputs = []               # other inputs
        
        self.lit_compt = 0             # for auxiliary variables generation
        
    def add_var(self, name):
        """
        add a logical variable to the dynamic system
        @param name: the name of the variable (string)
        """
        if name in self.base_var_set: 
            return
        else:
            self.base_var_set.add(name)
    
    def add_aux_var(self):
        """
        create an auxiliary variable for compiler purpose
        """
        name = '_lit' + str(self.lit_compt)
        self.lit_compt += 1
        self.base_var_set.add(name)
        return name
    
    def add_free_clock(self, hname):
        """
        add a free clock variable
        @param hname: the name of the clock variable (string)
        """
        if hname in self.base_var_set: 
            return
        else:
            self.base_var_set.add(hname)
            self.free_clocks.append(hname)
            self.symb_tab[hname] = ('clock', -1)
            
    def add_place_clock(self, pname, hname):
        """
        add an association hname --> pname between a clock and an inductive place
        """
        try:
            lpn = self.place_clocks[hname]
            lpn.append(pname)
        except:
            # clock not registered
            self.place_clocks[hname] = [pname]
            
    def add_input(self, name):
        """
        add a variable representing an input
        @param name: name of the input 
        """
        if not name in self.base_var_set:
            self.base_var_set.add(name)
        self.inputs.append(name)
            
    def add_clause(self, cla):
        """
        add a clause constraint
        """
        #TODO: do not add satisfied clauses (n or not n)
        self.list_clauses.append(cla)
        
    def add_aux_clause(self, cla):
        """
        add an auxiliary clause constraint
        """
        self.aux_list_clauses.append(cla)
            
    def get_var_number(self):
        """
        returns the number of variables used in the system (including aux vars)
        """
        return len(self.base_var_set)
    
            

    